@@ -1,9 +1,9 @@
 use darling::FromDeriveInput;
 use proc_macro2::{Ident, Span, TokenStream};
-use syn::{Data, DeriveInput, Path, Visibility};
+use syn::{parse_quote, Data, DeriveInput, Path, Visibility};
 
 /// Values we can parse from #[kube(attrs)]
-#[derive(Debug, Default, FromDeriveInput)]
+#[derive(Debug, FromDeriveInput)]
 #[darling(attributes(kube))]
 struct KubeAttrs {
     group: String,
@@ -33,11 +33,16 @@ struct KubeAttrs {
     printcolums: Vec<String>,
     #[darling(default)]
     scale: Option<String>,
+    #[darling(default = "default_crate")]
+    kube_core: Path,
 }
 
 fn default_apiext() -> String {
     "v1".to_owned()
 }
+fn default_crate() -> Path {
+    parse_quote! { ::kube::core } // by default must work well with people using facade crate
+}
 
 pub(crate) fn derive(input: proc_macro2::TokenStream) -> proc_macro2::TokenStream {
     let derive_input: DeriveInput = match syn::parse2(input) {
@@ -75,6 +80,7 @@ pub(crate) fn derive(input: proc_macro2::TokenStream) -> proc_macro2::TokenStrea
         printcolums,
         apiextensions,
         scale,
+        kube_core,
     } = kube_attrs;
 
     let struct_name = kind_struct.unwrap_or_else(|| kind.clone());
@@ -101,7 +107,7 @@ pub(crate) fn derive(input: proc_macro2::TokenStream) -> proc_macro2::TokenStrea
         field: status_field,
         default: status_default,
         impl_hasstatus,
-    } = process_status(&rootident, &status, &visibility);
+    } = process_status(&rootident, &status, &visibility, &kube_core);
     let has_status = status.is_some();
 
     let mut derive_paths: Vec<Path> = vec![];
@@ -159,8 +165,8 @@ pub(crate) fn derive(input: proc_macro2::TokenStream) -> proc_macro2::TokenStrea
         impl #rootident {
             pub fn new(name: &str, spec: #ident) -> Self {
                 Self {
-                    api_version: <#rootident as kube::Resource>::api_version(&()).to_string(),
-                    kind: <#rootident as kube::Resource>::kind(&()).to_string(),
+                    api_version: <#rootident as #kube_core::Resource>::api_version(&()).to_string(),
+                    kind: <#rootident as #kube_core::Resource>::kind(&()).to_string(),
                     metadata: k8s_openapi::apimachinery::pkg::apis::meta::v1::ObjectMeta {
                         name: Some(name.to_string()),
                         ..Default::default()
@@ -179,7 +185,7 @@ pub(crate) fn derive(input: proc_macro2::TokenStream) -> proc_macro2::TokenStrea
 
     let api_ver = format!("{}/{}", group, version);
     let impl_resource = quote! {
-        impl kube::Resource for #rootident {
+        impl #kube_core::Resource for #rootident {
             type DynamicType = ();
 
             fn group(_: &()) -> std::borrow::Cow<'_, str> {
@@ -218,8 +224,8 @@ pub(crate) fn derive(input: proc_macro2::TokenStream) -> proc_macro2::TokenStrea
             impl Default for #rootident {
                 fn default() -> Self {
                     Self {
-                        api_version: <#rootident as kube::Resource>::api_version(&()).to_string(),
-                        kind: <#rootident as kube::Resource>::kind(&()).to_string(),
+                        api_version: <#rootident as #kube_core::Resource>::api_version(&()).to_string(),
+                        kind: <#rootident as #kube_core::Resource>::kind(&()).to_string(),
                         metadata: k8s_openapi::apimachinery::pkg::apis::meta::v1::ObjectMeta::default(),
                         spec: Default::default(),
                         #status_default
@@ -247,7 +253,7 @@ pub(crate) fn derive(input: proc_macro2::TokenStream) -> proc_macro2::TokenStrea
         k8s_openapi::apiextensions_apiserver::pkg::apis::apiextensions::#v1ident
     };
     let extver = quote! {
-        kube::core::crd::#v1ident
+        #kube_core::crd::#v1ident
     };
 
     let shortnames_slice = {
@@ -373,8 +379,8 @@ pub(crate) fn derive(input: proc_macro2::TokenStream) -> proc_macro2::TokenStrea
                 #crd_meta_name
             }
 
-            fn api_resource() -> kube::core::ApiResource {
-                kube::core::ApiResource::erase::<Self>(&())
+            fn api_resource() -> #kube_core::dynamic::ApiResource {
+                #kube_core::dynamic::ApiResource::erase::<Self>(&())
             }
 
             fn shortnames() -> &'static [&'static str] {
@@ -383,7 +389,7 @@ pub(crate) fn derive(input: proc_macro2::TokenStream) -> proc_macro2::TokenStrea
         }
     };
 
-    let impl_hasspec = generate_hasspec(&ident, &rootident);
+    let impl_hasspec = generate_hasspec(&ident, &rootident, &kube_core);
 
     // Concat output
     quote! {
@@ -396,7 +402,7 @@ pub(crate) fn derive(input: proc_macro2::TokenStream) -> proc_macro2::TokenStrea
     }
 }
 
-/// This generates the code for the `kube::core::object::HasSpec` trait implementation.
+/// This generates the code for the `#kube_core::object::HasSpec` trait implementation.
 ///
 /// All CRDs have a spec so it is implemented for all of them.
 ///
@@ -404,9 +410,10 @@ pub(crate) fn derive(input: proc_macro2::TokenStream) -> proc_macro2::TokenStrea
 ///
 /// * `ident`: The identity (name) of the spec struct
 /// * `root ident`: The identity (name) of the main CRD struct (the one we generate in this macro)
-fn generate_hasspec(spec_ident: &Ident, root_ident: &Ident) -> TokenStream {
+/// * `kube_core`: The path stream for the analagous kube::core import location from users POV
+fn generate_hasspec(spec_ident: &Ident, root_ident: &Ident, kube_core: &Path) -> TokenStream {
     quote! {
-        impl ::kube::core::object::HasSpec for #root_ident {
+        impl #kube_core::object::HasSpec for #root_ident {
             type Spec = #spec_ident;
 
             fn spec(&self) -> &#spec_ident {
@@ -438,9 +445,15 @@ struct StatusInformation {
 /// * `root ident`: The identity (name) of the main CRD struct (the one we generate in this macro)
 /// * `status`: The optional name of the `status` struct to use
 /// * `visibility`: Desired visibility of the generated field
+/// * `kube_core`: The path stream for the analagous kube::core import location from users POV
 ///
 /// returns: A `StatusInformation` struct
-fn process_status(root_ident: &Ident, status: &Option<String>, visibility: &Visibility) -> StatusInformation {
+fn process_status(
+    root_ident: &Ident,
+    status: &Option<String>,
+    visibility: &Visibility,
+    kube_core: &Path,
+) -> StatusInformation {
     if let Some(status_name) = &status {
         let ident = format_ident!("{}", status_name);
         StatusInformation {
@@ -450,7 +463,7 @@ fn process_status(root_ident: &Ident, status: &Option<String>, visibility: &Visi
             },
             default: quote! { status: None, },
             impl_hasstatus: quote! {
-                impl ::kube::core::object::HasStatus for #root_ident {
+                impl #kube_core::object::HasStatus for #root_ident {
 
                     type Status = #ident;
 
